/* This library manipulates expiring firewall rules
 *
 * This library is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to:
 *      The Free Software Foundation, Inc.
 *      59 Temple Place, Suite 330
 *      Boston, MA  02111-1307  USA
 *
 * Copyright  2005 Bryan Cardillo <dillo@seas.upenn.edu>
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <getopt.h>
#include <time.h>

#include <iptables.h>
#include <linux/netfilter_ipv4/ipt_expire.h>

static void ipt_exp_help(void);
static int ipt_exp_parse(int, char **, int, unsigned int *,
		const struct ipt_entry *, unsigned int *,
		struct ipt_entry_match **);
static void ipt_exp_final_check(unsigned int);
static void ipt_exp_print(const struct ipt_ip *,
		const struct ipt_entry_match *, int);
static void ipt_exp_save(const struct ipt_ip *, const struct ipt_entry_match *);

/**
 * options
 */
static struct option ipt_exp_opts[] = {
	{ "expiration", 1, 0, 'e' },
	{ 0 }
};

/**
 * match
 */
static struct iptables_match ipt_expire_match = {
	.next = NULL,
	.name = "expire",
	.version = IPTABLES_VERSION,
	.size = IPT_ALIGN(sizeof(struct ipt_exp_info)),
	.userspacesize = IPT_ALIGN(sizeof(struct ipt_exp_info)),
	.help = &ipt_exp_help,
	.parse = &ipt_exp_parse,
	.final_check = &ipt_exp_final_check,
	.print = &ipt_exp_print,
	.save = &ipt_exp_save,
	.extra_opts = ipt_exp_opts
};

/**
 * shared library initialization
 * @see register_match()
 */
void
_init(void)
{
	register_match(&ipt_expire_match);
}

/**
 * print usage information
 */
static void
ipt_exp_help(void)
{
    printf("EXPIRE match options\n"
           "  --expiration [+]TIME\t\t"
	   "rule expires at [in] TIME\n\n");
}

/**
 * parse module specific options
 * @param c the short option character
 * @param argv the arguments array
 * @param invert is this an inverted argument
 * @param flags module specific flags
 * @param entry the entry
 * @param nfcache netfilter cache flags
 * @param match the match
 * @return zero if an option was found, non-zero otherwise
 */
static int
ipt_exp_parse(int c, char **argv, int invert, unsigned int *flags,
	const struct ipt_entry *entry, unsigned int *nfcache,
	struct ipt_entry_match **match)
{
	char *arg;
	struct ipt_exp_info *info;
	
	info = (struct ipt_exp_info *)(*match)->data;
	info->expiration = 0;
	switch (c) {
		case 'e':
			arg = argv[optind-1];
			check_inverse(arg, &invert, &optind, 0);
			if (invert)
				exit_error(PARAMETER_PROBLEM,
					"--expiration cannot be inverted");
                        if (*arg == '+')
				arg++;
			if (string_to_number_l(
					arg, 1, 0, &info->expiration) < 0)
				exit_error(PARAMETER_PROBLEM,
					"invalid expiration time");
			*flags = 1;
			if (*argv[optind-1] == '+')
				info->expiration += time(NULL);
			break;
		default:
			return 0;
	}
	return 1;
}

/**
 * ensures an expiration was specified
 * @param flags module specific flags from options parsing
 */
static void
ipt_exp_final_check(unsigned int flags)
{
	if (flags != 1)
		exit_error(PARAMETER_PROBLEM,
			"you must specify an expiration time (--expiration)");
}

/**
 * print information about an expiring match
 * in a format suitable for viewing
 * @param ip the address information
 * @param match the match
 * @param numeric the verbose level (?)
 */
static void
ipt_exp_print(const struct ipt_ip *ip,
	const struct ipt_entry_match *match, int numeric)
{
	struct ipt_exp_info *info;
	info = (struct ipt_exp_info *)match->data;
	printf("expires in %lds ", info->expiration - time(NULL));
}

/**
 * print information about an expiring match
 * in a format suitable for reconstructing the match
 * @param ip the address information
 * @param match the match
 */
static void
ipt_exp_save(const struct ipt_ip *ip, const struct ipt_entry_match *match)
{
	struct ipt_exp_info *info;
	info = (struct ipt_exp_info *)match->data;
	printf("-m expire --expiration %ld ", info->expiration);
}
